class UTFString;
class BList;

#ifndef UTFSTRINGTOKENIZER
#define UTFSTRINGTOKENIZER

#pragma export on

/*
	CLASS UTFStringTokenizer
	
	A helper class that allows the user to tokenize a given
	UTFString into a list o ftokens. Tokenizing is based upon both
	a delimiter string and a string of characters that are to 
	be ignored as possible tokens.
	
	Default for the delimiter is blank space
	Default for the ignore string is an empty string.
	
	The token string is chached for later re-use.
	
	HOW TO USE
	1) create a new UTFStringTokenizer
	2) Set the UTFString you wish to have tokenized.
	3) Set the specific delimiter you wish delinate the tokens by.
	4) Set the ignore string with the set of UTF Characters you wish to have
	ignored.
	
	Example 1: Default Usage
	
	UTFString tokens("This is a test!");
	UTFStringTokenizer st;
	BList tokens;
	
	st.SetTokenString(tokens)
	tokens = st.Tokenize;
	//Returns a list of tokens split by the space character
	//Token list includes the delimiter

	Example 2: Using a different Delimiter

	UTFString delim("is");
	
	st.SetDelimiterString(delim);
	tokens = st.Tokenize;
	//Returns a list of tokens split by the string is
	//Token list includes the delimiter

	Example 3: Setting ignore characters

	UTFString ignore(" ");
	delim = " ";
	
	st.SetDelimiterString(delim);
	st.SetIgnore(ignore);
	tokens = st.Tokenize;
	//Returns a list of tokens split by the space character
	//Token list does not includes the delimiter as the 
	//delimiter is part of the ignore character set
	
		
*/

class UTFStringTokenizer : public BLocker
{
public:
	//Constructor
	UTFStringTokenizer();
	
	//Destructor
	virtual ~UTFStringTokenizer();
	
	/*
		SetTokenString
		
		Sets the string to be tokenized.
		
		This string is cached for re-use.
	*/	
	void SetTokenString(const UTFString&);

	/*
		SetIgnoreString
		
		Sets the UTFString of characters that is to
		be ignored by the tokenizer. Any ignored character(s)
		is not placed in the returned list. This can include
		the Delimiter string.
	*/
	void SetIgnoreString(const UTFString&);

	/*
		SetDelimiterString
		
		Sets the delimiter string used by the tokenizer
		to tokenize the given string.
	*/
	void SetDelimiterString(const UTFString&);

	/*
		Tokenize
		
		Actually starts the tokenizing process. The set of tokems
		is then returned as a BList of UTFStrings. The list will
		contain the delimiter as well unless the delimiter is specified 
		in the ignore set.
	*/
	const BList& Tokenize();
		
protected:
private:

	void ClearTokens();
	
	//Token String
	UTFString m_strToken;
	
	//Ignore String
	UTFString m_strIgnore;
	
	//Delim String
	UTFString m_strDelim;
	
	//Token List
	BList	m_tokens;
	
};

#pragma export off

#endif