#include <stdio.h>
#include <stdlib.h>
#include "UTFChar.h"
#include "UTFString.h"
#include "UTFStringTokenizer.h"
#include "UTFTextBuffer.h"

void DumpTextBuffer(UTFTextBuffer& tb);

void main()
{


	BStopWatch* psw = new BStopWatch("UTF Library Timer Test");
	
	
	UTFString test;
	UTFString test2("This is a test!");
	UTFString test3(test2);
	

	printf("Constructor test\n");
	printf("const char*: %s\n", (const char*)test2);
	printf("UTFString: %s\n", (const char*)test3);
	
	printf("opeator= test\n");
	test = test2;
	printf("test: %s\n", (const char*)test);
	
	printf("BASIC String Test\n");

	test = test2.Left(3);
	printf("Left: %s\n", (const char*)test);

	test = test2.Right(3);
	printf("Right: %s\n", (const char*)test);

	test = test2.Mid(4,8);
	printf("Mid: %s\n", (const char*)test);
	
	printf("Substring Test\n");
	test = "is";
	printf("Sub Pos: %d\n",test2.InStr(test));
	

	printf("Replacement Test\n");
	test3 = "isn't";
	printf("Before: %s\n", (const char*)test2);
	test2.Replace(test, test3);
	printf("After: %s\n", (const char*)test2);
	
	printf("Insertion Test\n");
	test = "Insert";
	test2= "Test of Insertions Here";
	test3 = test2;
	test3.Insert(5,5,test);
	printf("Insert(5,5): %s\n", (const char*)test3);

	test3 = test2;
	test3.Insert(5,8,test);
	printf("Insert(5,8): %s\n", (const char*)test3);

	printf("Removal Test\n");
	test2= "Test of Insertion Here";
	test3 = test2;
	test3.Remove(4,5);
	printf("Remove(4,5): %s\n", (const char*)test3);

	test3 = test2;
	test3.Remove(4,8);
	printf("Remove(4,8): %s\n", (const char*)test3);
	
	printf("ToUpper Test\n");
	test3.ToUpper();
	printf("ToUpper: %s\n", (const char*)test3);

	printf("ToLower Test\n");
	test3.ToLower();
	printf("ToLower: %s\n", (const char*)test3);

	printf("String Tokenizer Tests\n");
	UTFStringTokenizer stok;
	BList tlist;
	
	test = "is";
	test2 = " !is";
	test3 = "This is a test!";

	printf("Default\n");
	stok.SetTokenString(test3);
	tlist = stok.Tokenize();
	
	UTFString *t;
	
	for (long i = 0; t = (UTFString*)tlist.ItemAt(i); i++)
	{
		printf("\tToken: %s\n", (const char*)*t);
	}
		
	printf("With delimiter %s\n", (const char*)test);

	stok.SetDelimiterString(test);
	tlist = stok.Tokenize();
	
	for (long i = 0; t = (UTFString*)tlist.ItemAt(i); i++)
	{
		printf("\tToken: %s\n", (const char*)*t);
	}

	printf("With ignore %s\n", (const char*)test2);

	stok.SetIgnoreString(test2);
	stok.SetDelimiterString(" ");
	tlist = stok.Tokenize();
	
	for (long i = 0; t = (UTFString*)tlist.ItemAt(i); i++)
	{
		printf("\tToken: %s\n", (const char*)*t);
	}
	
	printf ("Get/Set Char Test\n");
	
	UTFChar tc;
	
	for (long i = 0; i < test3.Length(); i++)
	{
		tc = test3.GetChar(i);
		printf("Char(%d): %s\n", i, (const char*)tc);	
	}

	tc = test3.GetChar(3);
	tc = "!";
	test3.SetChar(tc, 0);
	
	for (long i = 0; i < test3.Length(); i++)
	{
		tc = test3.GetChar(i);
		printf("Char(%d): %s\n", i, (const char*)tc);	
	}

	tc = "T";
	test3.SetChar(tc, test3.Length() - 1);
	
	for (long i = 0; i < test3.Length(); i++)
	{
		tc = test3.GetChar(i);
		printf("Char(%d): %s\n", i, (const char*)tc);	
	}

	
	
	BFile*	pFile = new BFile("UTFLib.txt",B_READ_ONLY);
	
	printf("readint in file UTFLib.txt\n");
	
	UTFTextBuffer tb(pFile);
	
	printf("***********************************************\n");
	printf("Created Text Buffer.\n");
	printf("Rows In Buffer: %d.\n", tb.NumRows());

	UTFString tt;
	DumpTextBuffer(tb);
	
	tb.SetSelectionBegin(10, 10);
	tb.SetSelectionEnd(12, 9);
	
	tt = tb.Copy();
	
	printf("\n\n***********************************************\n");
	printf("\n\nCopy Text: %s\n", (const char*)tt);
	
	tt = tb.Cut();
	
	printf("\n\n***********************************************\n");
	printf("Cut Text: %s\n", (const char*)tt);

	printf("\n\n***********************************************\n");
	DumpTextBuffer(tb);

	tb.SetSelectionBegin(10,0);
	tb.SetSelectionEnd(11,0);
	
	tb.Paste(tt);
	
	printf("\n\n***********************************************\n");
	printf("\n\nOaste Test\n\n");
	
	printf("\n\n***********************************************\n");
	DumpTextBuffer(tb);
	
	
	printf("\n\n***********************************************\n");
	printf("\n\nInsert Char Test\n");
	
	tb.SetCaret(0,0);
	
	for (uint32 i = 0; i < 2; i++)
	{
		UTFString t;
		t = tb.GetRow(i);
		printf("UnModified Row: %s", (const char*)t);
	}

	tb.InsertChar("A");
	tb.InsertChar("B");
	tb.InsertChar("C");
	tb.InsertChar("D");
	tb.InsertChar("\n");

	for (uint32 i = 0; i < 2; i++)
	{
		UTFString t;
		t = tb.GetRow(i);
		printf("Modified Row: %s", (const char*)t);
	}
	
	printf("\n\n***********************************************\n");
	printf("\n\nRemove Char Test\n");

	tb.SetCaret(0,0);
	
	for (uint32 i = 0; i < 2; i++)
	{
		UTFString tp;
		tp = tb.GetRow(i);
		printf("UnModified Row: %s", (const char*)tp);
	}

	tb.RemoveChar();
	tb.RemoveChar();
	tb.RemoveChar();
	tb.RemoveChar();

	for (uint32 i = 0; i < 2; i++)
	{
		UTFString tp;
		tp = tb.GetRow(i);
		printf("Modified Row: %s", (const char*)tp);
	}

	printf("\n\n***********************************************\n");
	printf("\n\nAddRow Test\n");
	DumpTextBuffer(tb);

	test = "This is a new row";
	
	tb.AddRow(test);
	tb.AddRow(tb.NumRows()-1, test);
	tb.AddRow(5, test);

	DumpTextBuffer(tb);

	printf("\n\n***********************************************\n");
	printf("\n\nRemove Test\n");

	tb.RemoveRow();
	tb.RemoveRow(tb.NumRows()-1);
	tb.RemoveRow(5);

	DumpTextBuffer(tb);

	printf("\n\n***********************************************\n");
	printf("\n\nGet/Set Row Test\n");

	test = tb.GetRow(10);
	
	test = "Blah B;ah!";
	
	tb.SetRow(10, test);
	DumpTextBuffer(tb);
	
	printf("\n\n***********************************************\n");
	printf("\n\nLoadData Test\n");

	pFile->Seek(0, SEEK_SET);
	tb.LoadData(pFile);
	DumpTextBuffer(tb);

	printf("\n\n***********************************************\n");
	printf("\n\noperator= Test\n");

	tb.ClearBuffer();
	pFile->Seek(0, SEEK_SET);
	UTFTextBuffer tb2(pFile);

	tb = tb2;
	DumpTextBuffer(tb);
	
	delete pFile;
	delete psw;

	exit(0);
}

void DumpTextBuffer(UTFTextBuffer& tb)
{
	for (uint32 rc = 0; rc < tb.NumRows(); rc++)
	{
		printf("%3d: %s", rc, (const char*)tb.GetRow(rc));
	}
}

