#include <string.h>
#include <ctype.h>

#include "UTFChar.h"

//Constructors
UTFChar::UTFChar()
{
	ClearBuf();
}

UTFChar::UTFChar(const char* tchar)
{
	ClearBuf();
	strncpy(m_buf, tchar, UTF8CHARBYTELEN(*tchar));
}

UTFChar::UTFChar(const UTFChar& tchar)
{
	ClearBuf();
	strcpy(m_buf, tchar.m_buf);
}
	
//Destructors
UTFChar::~UTFChar()
{
}

UTFChar& UTFChar::operator=(const char* tchar)
{
	BAutolock(this);
	
	ClearBuf();
	strncpy(m_buf, tchar, UTF8CHARBYTELEN(*tchar));
	return *this;
}

UTFChar& UTFChar::operator=(const UTFChar& tchar)
{
	
	BAutolock(this);
	if (0 == strcmp(m_buf, tchar.m_buf))
	{
		return *this;
	}
		
	ClearBuf();
	strcpy(m_buf, tchar.m_buf);
	return *this;
}

void UTFChar::ClearBuf()
{
	BAutolock(this);
	memset(m_buf, '\0', MAX_UTF_CHARLEN);
}

bool UTFChar::IsAlphaNum()
{
	
	BAutolock(this);
	
	if (m_buf[0] < 0x80)
	{
		return isalnum(m_buf[0]);	
	}
	else
	{
		return true;
	}
}

bool UTFChar::IsAscii()
{
	BAutolock(this);
	return (m_buf[0] < 0x80)?true:false;
}

bool UTFChar::IsNull()
{
	BAutolock(this);
	return m_buf[0]?false:true;
}

UTFChar::operator const char*() const
{
	BAutolock(this);
	return m_buf;
}

bool UTFChar::operator==(const char* tchar) const
{
	BAutolock(this);
	return (strcmp(tchar, m_buf))?false:true;
}

bool UTFChar::operator==(const UTFChar& tchar) const
{
	BAutolock(this);
	return (strcmp(tchar.m_buf, m_buf))?false:true;
}

bool UTFChar::operator!=(const char* tchar) const
{
	BAutolock(this);
	return !operator==(tchar);
}

bool UTFChar::operator!=(const UTFChar& tchar) const
{
	BAutolock(this);
	return !operator==(tchar);
}

uint8 UTFChar::ByteLength()
{
	
	BAutolock(this);
	return UTF8CHARBYTELEN(m_buf[0]);
}


